/*
   Function:  JUMPDEMO.PRG
   Author:    Greg Lief
   Dialect:   Clipper 5.01
   Copyright (c) 1992 Greg Lief
             c/o Grumpfish, Inc.
             2450 Lancaster Drive N.E., #206
             Salem, Oregon 97305 USA
             Tel 503.588.1815
             Fax 503.588.1980
             BBS 503.588.7572

   Compile:   clipper jumpdemo /n /w
   Link:      rtlink fi jumpdemo
              -or-
              blinker fi jumpdemo

   This source code may be used in your Clipper programs,
   provided that this file header and copyright notice
   remain intact.
*/

#include "box.ch"
#include "inkey.ch"

#xtranslate SCRNCENTER(<row>, <msg> [, <color> ] ) => ;
               SetPos( <row>, int(( maxcol()+1 - ;
                                    len(<msg>)) / 2)) ; ;
               DispOut( <msg> [, <color>] )

function jumpdemo(cdbf, cntx, startval, endval)
local x
local b
local searchval
local nfields
local key
local f
local msg
local oldcolor
local oldcursor
if cdbf == NIL .or. cntx == NIL
   ? "Syntax: JUMPDEMO <dbfname> <ntxname>" ;
     "[<startval> <endval>]"
   ? "For the purposes of this demo, the index" ;
     "key should be character type."
   ? "Optional parameters <startval> and" ;
     "<endval> restrict the data subset."
   return .f.
endif
oldcolor  := setcolor("n/bg, +w/r")
oldcursor := setcursor(0)
b := TBrowseDB(1, 1, maxrow()-1, maxcol()-1)
use (cdbf) index (cntx)

//----- create indexkey reference codeblock and
//----- store in b:cargo i-var
b:cargo := &("{ || " + indexkey(0) + "}")
if valtype(eval(b:cargo)) != "C"
   ? "This example is written specifically" ;
     "for character data..."
   ? "Please try it again with an index file" ;
     "containing a character index key"
   return .f.
endif

//----- create columns for each field in database
nfields := fcount()
for x := 1 to nfields
   f := field(x)
   b:AddColumn(TBColumnNew(f, fieldblock(f)))
next

//----- optional cosmetic niceties
b:colSep  := chr(32)  + chr(179) + chr(32)
b:headSep := chr(205) + chr(209) + chr(205)
scroll()
Left - 1, b:nBottom + 1, b:nRight + 1 box B_SINGLE + ' '

//----- tweak custom movement blocks if subset
//----- parameters were specified
if startval != NIL .and. endval != NIL
   searchval := substr(endval, 1, len(endval) - 1) + ;
                chr( asc(right(endval, 1)) + 1 )
   b:goTopBlock := { || dbseek(startval, .t.) }
   b:goBottomBlock := { || dbseek(searchval, .t.), ;
                           dbskip(-1) }
   b:skipBlock := { | skipCnt | ;
                      gilligan(skipCnt, b:cargo, ;
                               startval, endval) }
   eval(b:goTopBlock)
   ScrnCenter(b:nTop - 1, "[ Viewing data between " + ;
              startval + " and " + endval + " ]", "+w/bg")
endif
ScrnCenter(b:nBottom + 1, ;
           "[ Press HOME and END to jump to start " + ;
           "of previous/next subgroup ]", "+w/bg")

//----- da main loop...
do while key != K_ESC
   dispbegin()
   do while ( key := inkey() ) == 0 .and. ! b:stabilize()
   enddo
   dispend()
   if b:stable
      key := inkey(0)
   endif
   do case
      case key == K_UP
         b:up()
         b:stabilize()
         if b:hitTop
            alert("Top of data")
         endif
      case key == K_DOWN
         b:down()
         b:stabilize()
         if b:hitBottom
            alert("Bottom of data")
         endif
      case key == K_PGUP
         b:pageUp()
      case key == K_PGDN
         b:pageDown()
      case key == K_CTRL_PGUP
         b:goTop()
      case key == K_CTRL_PGDN
         b:goBottom()
      case key == K_HOME .and. b:stable
         JumpUp(b, startval)
      case key == K_END .and. b:stable
         JumpDown(b, endval)
      case key == K_LEFT
         b:left()
      case key == K_RIGHT
         b:right()
   endcase
enddo
use
setcolor(oldcolor)
setcursor(oldcursor)
return nil

/*
   Gilligan(): custom skipBlock function for use with data subsets
*/
static function gilligan(skipcnt, val, startval, endval)
local movement := 0
do case
   /* no movement */
   case skipcnt == 0
      skip 0
   /* moving forward */
   case skipcnt > 0
      do while movement < skipcnt .and. eval(val) <= endval
         skip 1
         movement++
      enddo
      // Make sure that we are within range.
      // If not, move backward
      do while (eval(val) > endval .or. ;
                eof()) .and. ! bof()
         skip -1
         movement--
      enddo
      if bof()         // no data in range... fall out
         keyboard chr(K_ESC)
      endif
   /* moving backward */
   case skipcnt < 0
      do while movement > skipcnt .and. ;
               eval(val) >= startval
         skip -1
         if bof()
            exit
         endif
         movement--
      enddo
      // Make sure that we are within range.
      // If not, move forward.
      do while eval(val) < startval .and. ! eof()
         skip
         movement++
      enddo
      if eof()         // no data within range... fall out
         keyboard chr(K_ESC)
      endif
endcase
return movement

//----- end static function Gilligan()
//-------------------------------------------------------//

/*
   JumpDown(): jump to beginning of next subgroup (active index only!)
*/
static function jumpdown(b, endval)
local val := eval(b:cargo)
local marker := recno()
//----- only perform the jump if we are not passing
//----- the end of the data subset
if endval == NIL .or. endval >= val
   dbseek( substr(val, 1, len(val) - 1) + ;
           chr( asc(right(val, 1)) + 1 ), .t.)
   if eof()
      go marker
   else
      b:refreshAll()
   endif
endif
return nil

//----- end static function JumpDown()
//-------------------------------------------------------//

/*
   JumpUp(): jump to beginning of previous subgroup (active index only!)
*/
static function jumpup(b, startval)
local val
//----- only perform the jump if we are not passing the
//----- start of the data subset
if startval == NIL .or. eval(b:cargo) >> startval
   dbseek(eval(b:cargo), .t.)
   dbskip(-1)
   val := eval(b:cargo)
   b:goTop()
   dbseek(val, .t.)
   b:refreshAll()
endif
return nil

//----- end static function JumpUp()
//-------------------------------------------------------//

